@@ -30,10 +30,6 @@
 #include <raptor_config.h>
 #endif
 
-#ifdef WIN32
-#include <win32_raptor_config.h>
-#endif
-
 #include <stdio.h>
 #include <string.h>
 #include <ctype.h>
@@ -57,7 +53,7 @@
 
 
 /* Prototypes for local functions */
-static void raptor_ntriples_generate_statement(raptor_parser* parser, const unsigned char *subject, const raptor_term_type subject_type, const unsigned char *predicate, const raptor_term_type predicate_type, const void *object, const raptor_term_type object_type, const unsigned char *object_literal_language, const unsigned char *object_literal_datatype, const void *graph, const raptor_term_type graph_type);
+static void raptor_ntriples_generate_statement(raptor_parser* parser, raptor_term* subject_term, raptor_term* predicate_term, raptor_term* object_term, raptor_term* graph_term);
 
 /*
  * NTriples parser object
@@ -66,9 +62,9 @@ struct raptor_ntriples_parser_context_s {
   /* current line */
   unsigned char *line;
   /* current line length */
-  int line_length;
+  size_t line_length;
   /* current char in line buffer */
-  int offset;
+  size_t offset;
 
   char last_char;
   
@@ -129,21 +125,13 @@ raptor_ntriples_parse_terminate(raptor_parser* rdf_parser)
 
 static void
 raptor_ntriples_generate_statement(raptor_parser* parser, 
-                                   const unsigned char *subject,
-                                   const raptor_term_type subject_type,
-                                   const unsigned char *predicate,
-                                   const raptor_term_type predicate_type,
-                                   const void *object,
-                                   const raptor_term_type object_type,
-                                   const unsigned char *object_literal_language,
-                                   const unsigned char *object_literal_datatype,
-                                   const void *graph,
-                                   const raptor_term_type graph_type)
+                                   raptor_term *subject,
+                                   raptor_term *predicate,
+                                   raptor_term *object,
+                                   raptor_term *graph)
 {
   /* raptor_ntriples_parser_context *ntriples_parser = (raptor_ntriples_parser_context*)parser->context; */
   raptor_statement *statement = &parser->statement;
-  raptor_uri *predicate_uri = NULL;
-  raptor_uri *datatype_uri = NULL;
 
   if(!parser->emitted_default_graph) {
     raptor_parser_start_graph(parser, NULL, 0);
@@ -154,107 +142,16 @@ raptor_ntriples_generate_statement(raptor_parser* parser,
   if(!parser->statement_handler)
     goto cleanup;
 
-  /* Two choices for subject from N-Triples */
-  if(subject_type == RAPTOR_TERM_TYPE_BLANK) {
-    statement->subject = raptor_new_term_from_blank(parser->world, subject);
-  } else {
-    raptor_uri *subject_uri;
-
-    /* must be RAPTOR_TERM_TYPE_URI */
-    subject_uri = raptor_new_uri(parser->world, subject);
-    if(!subject_uri) {
-      raptor_parser_error(parser, "Could not create subject uri '%s', skipping", subject);
-      goto cleanup;
-    }
-    statement->subject = raptor_new_term_from_uri(parser->world, subject_uri);
-    raptor_free_uri(subject_uri);
-  }
-
-  if(object_literal_datatype) {
-    datatype_uri = raptor_new_uri(parser->world, object_literal_datatype);
-    if(!datatype_uri) {
-      raptor_parser_error(parser, "Could not create object literal datatype uri '%s', skipping", object_literal_datatype);
-      goto cleanup;
-    }
-    object_literal_language = NULL;
-  }
-
-  /* Predicates in N-Triples are URIs but check for bad ordinals */
-  if(!strncmp((const char*)predicate, "http://www.w3.org/1999/02/22-rdf-syntax-ns#_", 44)) {
-    int predicate_ordinal = raptor_check_ordinal(predicate+44);
-    if(predicate_ordinal <= 0)
-      raptor_parser_error(parser, "Illegal ordinal value %d in property '%s'.", predicate_ordinal, predicate);
-  }
-  
-  predicate_uri = raptor_new_uri(parser->world, predicate);
-  if(!predicate_uri) {
-    raptor_parser_error(parser, "Could not create predicate uri '%s', skipping", predicate);
-    goto cleanup;
-  }
-  statement->predicate = raptor_new_term_from_uri(parser->world, predicate_uri);
-  raptor_free_uri(predicate_uri);
-  predicate_uri = NULL;
-  
-  /* Three choices for object from N-Triples */
-  if(object_type == RAPTOR_TERM_TYPE_URI) {
-    raptor_uri *object_uri;
-
-    object_uri = raptor_new_uri(parser->world, (const unsigned char*)object);
-    if(!object_uri) {
-      raptor_parser_error(parser, "Could not create object uri '%s', skipping", (const char *)object);
-      goto cleanup;
-    }
-    statement->object = raptor_new_term_from_uri(parser->world, object_uri);
-    raptor_free_uri(object_uri);
-    object_uri = NULL;
-  } else if(object_type == RAPTOR_TERM_TYPE_BLANK) {
-    statement->object = raptor_new_term_from_blank(parser->world, 
-                                                   (const unsigned char*)object);
-  } else { 
-    /*  RAPTOR_TERM_TYPE_LITERAL */
-    statement->object = raptor_new_term_from_literal(parser->world,
-                                                     (const unsigned char*)object,
-                                                     datatype_uri,
-                                                     (const unsigned char*)object_literal_language);
-  }
-
-
-  if(graph) {
-    /* Three choices for graph/context from N-Quads according to
-     * http://sw.deri.org/2008/07/n-quads/ but I am IGNORING Literal
-     */
-    if(graph_type == RAPTOR_TERM_TYPE_URI) {
-      raptor_uri *graph_uri;
-
-      graph_uri = raptor_new_uri(parser->world, (const unsigned char*)graph);
-      if(!graph_uri) {
-        raptor_parser_error(parser,
-                            "Could not create object uri '%s', skipping", 
-                            (const char *)graph);
-        goto cleanup;
-      }
-      statement->graph = raptor_new_term_from_uri(parser->world, graph_uri);
-      raptor_free_uri(graph_uri);
-      graph_uri = NULL;
-    } else if(graph_type == RAPTOR_TERM_TYPE_BLANK) {
-      statement->graph = raptor_new_term_from_blank(parser->world, 
-                                                    (const unsigned char*)graph);
-    } else { 
-      /* Warning about literal graphs is handled below */
-      statement->graph = NULL;
-    }
-  }
+  statement->subject = subject;
+  statement->predicate = predicate;
+  statement->object = object;
+  statement->graph = graph;
 
   /* Generate the statement */
   (*parser->statement_handler)(parser->user_data, statement);
 
   cleanup:
   raptor_free_statement(statement);
-
-  if(predicate_uri)
-    raptor_free_uri(predicate_uri);
-  if(datatype_uri)
-    raptor_free_uri(datatype_uri);
 }
 
 
@@ -269,8 +166,7 @@ typedef enum {
   RAPTOR_TERM_CLASS_URI,      /* ends on > */
   RAPTOR_TERM_CLASS_BNODEID,  /* ends on first non [A-Za-z][A-Za-z0-9]* */
   RAPTOR_TERM_CLASS_STRING,   /* ends on non-escaped " */
-  RAPTOR_TERM_CLASS_LANGUAGE, /* ends on first non [a-z0-9]+ ('-' [a-z0-9]+ )? */
-  RAPTOR_TERM_CLASS_FULL      /* the entire string is used */
+  RAPTOR_TERM_CLASS_LANGUAGE  /* ends on first non [a-z0-9]+ ('-' [a-z0-9]+ )? */
 } raptor_ntriples_term_class;
 
 
@@ -300,14 +196,10 @@ raptor_ntriples_term_valid(raptor_parser* rdf_parser,
       break;
 
     case RAPTOR_TERM_CLASS_LANGUAGE:
-      /* ends on first non [a-z0-9]+ ('-' [a-z0-9]+ )? */
-      result = (IS_ASCII_ALPHA(c) || IS_ASCII_DIGIT(c));
+      /* ends on first non [a-zA-Z]+ ('-' [a-zA-Z0-9]+ )? */
+      result = IS_ASCII_ALPHA(c);
       if(position)
-        result = (result || c == '-');
-      break;
-      
-    case RAPTOR_TERM_CLASS_FULL:
-      result = 1;
+        result = (result || IS_ASCII_DIGIT(c) || c == '-');
       break;
       
     default:
@@ -328,7 +220,6 @@ raptor_ntriples_term_valid(raptor_parser* rdf_parser,
  * @dest_lenp: pointer to length of destination string (out)
  * @end_char: string ending character
  * @class: string class
- * @allow_utf8: Non-0 if UTF-8 chars are allowed in the term
  *
  * Parse an N-Triples term with escapes.
  *
@@ -341,11 +232,7 @@ raptor_ntriples_term_valid(raptor_parser* rdf_parser,
  * outside the printable ASCII range are discarded with a warning.
  * See the grammar for full details of the allowed ranges.
  *
- * If the class is RAPTOR_TERM_CLASS_FULL, the end_char is ignored.
- *
- * UTF-8 is only allowed if allow_utf8 is non-0, otherwise the
- * string is US-ASCII and only the \u and \U esapes are allowed.
- * If enabled, both are allowed.
+ * UTF-8 and the \u and \U esapes are both allowed.
  *
  * Return value: Non 0 on failure
  **/
@@ -354,8 +241,7 @@ raptor_ntriples_term(raptor_parser* rdf_parser,
                      const unsigned char **start, unsigned char *dest, 
                      size_t *lenp, size_t *dest_lenp,
                      char end_char,
-                     raptor_ntriples_term_class term_class,
-                     int allow_utf8)
+                     raptor_ntriples_term_class term_class)
 {
   const unsigned char *p = *start;
   unsigned char c = '\0';
@@ -364,11 +250,10 @@ raptor_ntriples_term(raptor_parser* rdf_parser,
   unsigned int position = 0;
   int end_char_seen = 0;
 
-  if(term_class == RAPTOR_TERM_CLASS_FULL)
-    end_char = '\0';
-  
   /* find end of string, fixing backslashed characters on the way */
   while(*lenp > 0) {
+    int unichar_width;
+
     c = *p;
 
     p++;
@@ -376,32 +261,24 @@ raptor_ntriples_term(raptor_parser* rdf_parser,
     rdf_parser->locator.column++;
     rdf_parser->locator.byte++;
 
-    if(allow_utf8) {
-      if(c > 0x7f) {
-        /* just copy the UTF-8 bytes through */
-        size_t unichar_len;
-        unichar_len = raptor_unicode_utf8_string_get_char(p - 1, 1 + *lenp, NULL);
-        if(unichar_len > *lenp) {
-          raptor_parser_error(rdf_parser, "UTF-8 encoding error at character %d (0x%02X) found.", c, c);
-          /* UTF-8 encoding had an error or ended in the middle of a string */
-          return 1;
-        }
-        memcpy(dest, p-1, unichar_len);
-        dest += unichar_len;
-
-        unichar_len--; /* p, *lenp were moved on by 1 earlier */
-        
-        p += unichar_len;
-        (*lenp) -= unichar_len;
-        rdf_parser->locator.column += unichar_len;
-        rdf_parser->locator.byte += unichar_len;
-        continue;
+    if(c > 0x7f) {
+      /* just copy the UTF-8 bytes through */
+      int unichar_len;
+      unichar_len = raptor_unicode_utf8_string_get_char(p - 1, 1 + *lenp, NULL);
+      if(unichar_len < 0 || RAPTOR_GOOD_CAST(size_t, unichar_len) > *lenp) {
+        raptor_parser_error(rdf_parser, "UTF-8 encoding error at character %d (0x%02X) found.", c, c);
+        /* UTF-8 encoding had an error or ended in the middle of a string */
+        return 1;
       }
-    } else if(!IS_ASCII_PRINT(c)) {
-      /* This is an ASCII check, not a printable character check 
-       * so isprint() is not appropriate, since that is a locale check.
-       */
-      raptor_parser_error(rdf_parser, "Non-printable ASCII character %d (0x%02X) found.", c, c);
+      memmove(dest, p-1, unichar_len);
+      dest += unichar_len;
+
+      unichar_len--; /* p, *lenp were moved on by 1 earlier */
+      
+      p += unichar_len;
+      (*lenp) -= unichar_len;
+      rdf_parser->locator.column += unichar_len;
+      rdf_parser->locator.byte += unichar_len;
       continue;
     }
     
@@ -434,8 +311,7 @@ raptor_ntriples_term(raptor_parser* rdf_parser,
     }
 
     if(!*lenp) {
-      if(term_class != RAPTOR_TERM_CLASS_FULL)
-        raptor_parser_error(rdf_parser, "\\ at end of line");
+      raptor_parser_error(rdf_parser, "\\ at end of line");
       return 0;
     }
 
@@ -460,6 +336,17 @@ raptor_ntriples_term(raptor_parser* rdf_parser,
       case 't':
         *dest++ = '\t';
         break;
+      case '<':
+      case '>':
+      case '{':
+      case '}':
+      case '|':
+      case '^':
+      case '`':
+        /* Turtle 2013 allows these in URIs (as well as \" and \\) */
+        *dest++ = c;
+        break;
+
       case 'u':
       case 'U':
         ulen = (c == 'u') ? 4 : 8;
@@ -470,8 +357,23 @@ raptor_ntriples_term(raptor_parser* rdf_parser,
         }
 
         if(1) {
-          int n;
+          unsigned int ii;
+          int n = 0;
 
+          for(ii = 0; ii < ulen; ii++) {
+            char cc = p[ii];
+            if(!isxdigit(RAPTOR_GOOD_CAST(char, cc))) {
+              raptor_parser_error(rdf_parser,
+                            "N-Triples string error - illegal hex digit %c in Unicode escape '%c%s...'",
+                            cc, c, p);
+              n = 1;
+              break;
+            }
+          }
+
+          if(n)
+            break;
+          
           n = sscanf((const char*)p, ((ulen == 4) ? "%04lx" : "%08lx"), &unichar);
           if(n != 1) {
             raptor_parser_error(rdf_parser, "Illegal Uncode escape '%c%s...'", c, p);
@@ -481,8 +383,8 @@ raptor_ntriples_term(raptor_parser* rdf_parser,
 
         p += ulen;
         (*lenp) -= ulen;
-        rdf_parser->locator.column += ulen;
-        rdf_parser->locator.byte += ulen;
+        rdf_parser->locator.column += RAPTOR_GOOD_CAST(int, ulen);
+        rdf_parser->locator.byte += RAPTOR_GOOD_CAST(int, ulen);
         
         if(unichar > raptor_unicode_max_codepoint) {
           raptor_parser_error(rdf_parser,
@@ -491,13 +393,21 @@ raptor_ntriples_term(raptor_parser* rdf_parser,
           break;
         }
 
+        unichar_width = raptor_unicode_utf8_string_put_char(unichar, dest, 4);
+        if(unichar_width < 0) {
+          raptor_parser_error(rdf_parser,
+                              "Illegal Unicode character with code point #x%lX.", 
+                              unichar);
+          break;
+        }
+
         /* The destination length is set here to 4 since we know that in 
          * all cases, the UTF-8 encoded output sequence is always shorter
          * than the input sequence, and the buffer is edited in place.
          *   \uXXXX: 6 bytes input - UTF-8 max 3 bytes output
          *   \uXXXXXXXX: 10 bytes input - UTF-8 max 4 bytes output
          */
-        dest += raptor_unicode_utf8_string_put_char(unichar, dest, 4);
+        dest += (int)unichar_width;
         break;
 
       default:
@@ -539,12 +449,8 @@ raptor_ntriples_parse_line(raptor_parser* rdf_parser,
   int i;
   unsigned char *p;
   unsigned char *dest;
-  unsigned char *terms[MAX_NTRIPLES_TERMS] = { NULL, NULL, NULL, NULL};
-  size_t term_lengths[MAX_NTRIPLES_TERMS] = {0, 0, 0, 0};
-  raptor_term_type term_types[MAX_NTRIPLES_TERMS] = {RAPTOR_TERM_TYPE_UNKNOWN, RAPTOR_TERM_TYPE_UNKNOWN, RAPTOR_TERM_TYPE_UNKNOWN, RAPTOR_TERM_TYPE_UNKNOWN};
+  raptor_term* real_terms[MAX_NTRIPLES_TERMS] = {NULL, NULL, NULL, NULL};
   size_t term_length = 0;
-  unsigned char *object_literal_language = NULL;
-  unsigned char *object_literal_datatype = NULL;
   int rc = 0;
   
   /* ASSERTION:
@@ -585,23 +491,13 @@ raptor_ntriples_parse_line(raptor_parser* rdf_parser,
 
   /* can't be empty now - that would have been caught above */
   
-  /* Check for terminating '.' */
-  if(p[len-1] != '.') {
-    /* Move current location to point to problem */
-    rdf_parser->locator.column += len-2;
-    rdf_parser->locator.byte += len-2;
-    raptor_parser_error(rdf_parser, "Missing . at end of line");
-    return 0;
-  }
+  /* Must be triple/quad */
 
-  p[len-1] = '\0';
-  len--;
-
-
-  /* Must be triple */
-
-  for(i = 0; i < max_terms; i++) {
+  for(i = 0; i < 4; i++) {
     if(!len) {
+      /* context is optional in nquads */
+      if (i == 3)
+	break;
       raptor_parser_error(rdf_parser, "Unexpected end of line");
       goto cleanup;
     }
@@ -630,8 +526,6 @@ raptor_ntriples_parse_line(raptor_parser* rdf_parser,
 
     switch(*p) {
       case '<':
-        term_types[i] = RAPTOR_TERM_TYPE_URI;
-        
         dest = p;
 
         p++;
@@ -642,15 +536,43 @@ raptor_ntriples_parse_line(raptor_parser* rdf_parser,
         if(raptor_ntriples_term(rdf_parser,
                                 (const unsigned char**)&p, 
                                 dest, &len, &term_length, 
-                                '>', RAPTOR_TERM_CLASS_URI, 0)) {
+                                '>', RAPTOR_TERM_CLASS_URI)) {
           rc = 1;
           goto cleanup;
         }
+
+        if(!raptor_turtle_check_uri_string(dest)) {
+          raptor_parser_error(rdf_parser, "URI '%s' contains bad character(s)", dest);
+          rc = 1;
+          goto cleanup;
+        }
+
+        if(1) {
+          raptor_uri *uri;
+
+          /* Check for bad ordinal predicate */
+          if(!strncmp((const char*)dest,
+                      "http://www.w3.org/1999/02/22-rdf-syntax-ns#_", 44)) {
+            int ordinal = raptor_check_ordinal(dest + 44);
+            if(ordinal <= 0)
+              raptor_parser_error(rdf_parser, "Illegal ordinal value %d in property '%s'.", ordinal, dest);
+          }
+          if(raptor_uri_uri_string_is_absolute(dest) <= 0) {
+            raptor_parser_error(rdf_parser, "URI '%s' is not absolute.", dest);
+            goto cleanup;
+          }
+          
+          uri = raptor_new_uri(rdf_parser->world, dest);
+          if(!uri) {
+            raptor_parser_error(rdf_parser, "Could not create URI for '%s'", (const char *)dest);
+            goto cleanup;
+          }
+          real_terms[i] = raptor_new_term_from_uri(rdf_parser->world, uri);
+          raptor_free_uri(uri);
+        }
         break;
 
       case '"':
-        term_types[i] = RAPTOR_TERM_TYPE_LITERAL;
-        
         dest = p;
 
         p++;
@@ -661,79 +583,119 @@ raptor_ntriples_parse_line(raptor_parser* rdf_parser,
         if(raptor_ntriples_term(rdf_parser,
                                 (const unsigned char**)&p,
                                 dest, &len, &term_length,
-                                '"', RAPTOR_TERM_CLASS_STRING, 0)) {
+                                '"', RAPTOR_TERM_CLASS_STRING)) {
           rc = 1;
           goto cleanup;
         }
-        
-        if(len && (*p == '-' || *p == '@')) {
-          if(*p == '-')
-            raptor_parser_error(rdf_parser, "Old N-Triples language syntax using \"string\"-lang rather than \"string\"@lang.");
 
-          object_literal_language = p;
+        if(1) {
+          unsigned char *object_literal_language = NULL;
+          unsigned char *object_literal_datatype = NULL;
+          raptor_uri* datatype_uri = NULL;
+    
+          if(len && *p == '@') {
+            unsigned char *q;
+            size_t lang_len;
+            
+            object_literal_language = p;
 
-          /* Skip - */
-          p++;
-          len--;
-          rdf_parser->locator.column++;
-          rdf_parser->locator.byte++;
+            /* Skip - */
+            p++;
+            len--;
+            rdf_parser->locator.column++;
+            rdf_parser->locator.byte++;
 
-          if(!len) {
-            raptor_parser_error(rdf_parser, "Missing language after \"string\"-");
-            goto cleanup;
+            if(!len) {
+              raptor_parser_error(rdf_parser, "Missing language after \"string\"-");
+              goto cleanup;
+            }
+
+
+            if(raptor_ntriples_term(rdf_parser,
+                                    (const unsigned char**)&p,
+                                    object_literal_language, &len, &lang_len,
+                                    '\0', RAPTOR_TERM_CLASS_LANGUAGE)) {
+              rc = 1;
+              goto cleanup;
+            }
+
+            if(!lang_len) {
+              raptor_parser_error(rdf_parser, "Invalid language tag at @%s", p);
+              rc = 1;
+              goto cleanup;
+            }
+            
+            /* Normalize language to lowercase
+             * http://www.w3.org/TR/rdf-concepts/#dfn-language-identifier
+             */
+            for(q = object_literal_language; *q; q++) {
+              if(IS_ASCII_UPPER(*q))
+                *q = TO_ASCII_LOWER(*q);
+            }
+
+          }
+
+          if(len >1 && *p == '^' && p[1] == '^') {
+
+            object_literal_datatype = p;
+
+            /* Skip ^^ */
+            p += 2;
+            len -= 2;
+            rdf_parser->locator.column += 2;
+            rdf_parser->locator.byte += 2;
+
+            if(!len || (len && *p != '<')) {
+              raptor_parser_error(rdf_parser, "Missing datatype URI-ref in\"string\"^^<URI-ref> after ^^");
+              goto cleanup;
+            }
+
+            p++;
+            len--;
+            rdf_parser->locator.column++;
+            rdf_parser->locator.byte++;
+
+            if(raptor_ntriples_term(rdf_parser,
+                                    (const unsigned char**)&p,
+                                    object_literal_datatype, &len, NULL,
+                                    '>', RAPTOR_TERM_CLASS_URI)) {
+              rc = 1;
+              goto cleanup;
+            }
+
+            if(raptor_uri_uri_string_is_absolute(object_literal_datatype) <= 0) {
+              raptor_parser_error(rdf_parser, "Datatype URI '%s' is not absolute.", object_literal_datatype);
+              rc = 1;
+              goto cleanup;
+            }
+
+          }
+
+          if(object_literal_datatype && object_literal_language) {
+            raptor_parser_warning(rdf_parser, "Typed literal used with a language - ignoring the language");
+            object_literal_language = NULL;
+          }
+
+          if(object_literal_datatype) {
+            datatype_uri = raptor_new_uri(rdf_parser->world,
+                                          object_literal_datatype);
+            if(!datatype_uri) {
+              raptor_parser_error(rdf_parser, "Could not create literal datatype uri '%s'", object_literal_datatype);
+              goto cleanup;
+            }
+            object_literal_language = NULL;
           }
           
-
-          if(raptor_ntriples_term(rdf_parser,
-                                  (const unsigned char**)&p,
-                                  object_literal_language, &len, NULL,
-                                  '\0', RAPTOR_TERM_CLASS_LANGUAGE, 0)) {
-            rc = 1;
-            goto cleanup;
-          }
-        }
-
-        if(len >1 && *p == '^' && p[1] == '^') {
-
-          object_literal_datatype = p;
-
-          /* Skip ^^ */
-          p += 2;
-          len -= 2;
-          rdf_parser->locator.column += 2;
-          rdf_parser->locator.byte += 2;
-
-          if(!len || (len && *p != '<')) {
-            raptor_parser_error(rdf_parser, "Missing datatype URI-ref in\"string\"^^<URI-ref> after ^^");
-            goto cleanup;
-          }
-
-          p++;
-          len--;
-          rdf_parser->locator.column++;
-          rdf_parser->locator.byte++;
-
-          if(raptor_ntriples_term(rdf_parser,
-                                  (const unsigned char**)&p,
-                                  object_literal_datatype, &len, NULL,
-                                  '>', RAPTOR_TERM_CLASS_URI, 0)) {
-            rc = 1;
-            goto cleanup;
-          }
-          
-        }
-
-        if(object_literal_datatype && object_literal_language) {
-          raptor_parser_warning(rdf_parser, "Typed literal used with a language - ignoring the language");
-          object_literal_language = NULL;
+          real_terms[i] = raptor_new_term_from_literal(rdf_parser->world,
+                                                       dest,
+                                                       datatype_uri,
+                                                       object_literal_language);
         }
 
         break;
 
 
       case '_':
-        term_types[i] = RAPTOR_TERM_TYPE_BLANK;
-
         /* store where _ was */
         dest = p;
 
@@ -757,7 +719,7 @@ raptor_ntriples_parse_line(raptor_parser* rdf_parser,
         if(raptor_ntriples_term(rdf_parser,
                                 (const unsigned char**)&p,
                                 dest, &len, &term_length,
-                                '\0', RAPTOR_TERM_CLASS_BNODEID, 0)) {
+                                '\0', RAPTOR_TERM_CLASS_BNODEID)) {
           rc = 1;
           goto cleanup;
         }
@@ -767,6 +729,8 @@ raptor_ntriples_parse_line(raptor_parser* rdf_parser,
           goto cleanup;
         }
 
+        real_terms[i] = raptor_new_term_from_blank(rdf_parser->world, dest);
+
         break;
 
       default:
@@ -776,12 +740,9 @@ raptor_ntriples_parse_line(raptor_parser* rdf_parser,
     }
 
 
-    /* Store term */
-    terms[i] = dest; term_lengths[i] = term_length;
-
     /* Whitespace must separate the terms */
     if(i < 2 && !isspace((int)*p)) {
-      raptor_parser_error(rdf_parser, "Missing whitespace after term '%s'", terms[i]);
+      raptor_parser_error(rdf_parser, "Missing whitespace after term");
       rc = 1;
       goto cleanup;
     }
@@ -795,49 +756,71 @@ raptor_ntriples_parse_line(raptor_parser* rdf_parser,
     }
 
 #if defined(RAPTOR_DEBUG) && RAPTOR_DEBUG > 1
-    fprintf(stderr, "item %d: term '%s' len %d type %d\n",
-            i, terms[i], (unsigned int)term_lengths[i], term_types[i]);
-#endif    
-  }
+    if(1) {
+      unsigned char* c = raptor_term_to_string(real_terms[i]);
+      fprintf(stderr, "item %d: term '%s' type %d\n",
+              i, c, real_terms[i]->type);
+      raptor_free_string(c);
+    }
+#endif
 
-  if(len) {
-    raptor_parser_error(rdf_parser, "Junk before terminating \".\"");
-    return 0;
-  }
-  
+    /* Look for terminating '.' after 3rd (ntriples) or 3rd/4th (nquads) term */
+    if(i == (ntriples_parser->is_nquads ? 4 : 3) && *p != '.') {
+      raptor_parser_error(rdf_parser, "Missing terminating \".\"");
+      return 0;
+    }
 
-  if(object_literal_language) {
-    unsigned char *q;
-    /* Normalize language to lowercase
-     * http://www.w3.org/TR/rdf-concepts/#dfn-language-identifier
-     */
-    for(q = object_literal_language; *q; q++) {
-      if(IS_ASCII_UPPER(*q))
-        *q = TO_ASCII_LOWER(*q);
+    /* Still may be optional so check again */
+    if(*p == '.') {
+      p++;
+      len--;
+      rdf_parser->locator.column++;
+      rdf_parser->locator.byte++;
+
+      /* Skip whitespace after '.' */
+      while(len > 0 && isspace((int)*p)) {
+        p++;
+        len--;
+        rdf_parser->locator.column++;
+        rdf_parser->locator.byte++;
+      }
+
+      /* Only a comment is allowed here */
+      if(*p && *p != '#') {
+        raptor_parser_error(rdf_parser, "Junk after terminating \".\"");
+        return 0;
+      }
+
+      p += len; len = 0;
     }
   }
 
 
-  /* Just to be sure */
-  if(!ntriples_parser->is_nquads)
-    terms[3] = NULL;
+  /* Check N-Triples has only 3 terms */
+  if(!ntriples_parser->is_nquads) {
+    if(real_terms[3]) {
+      raptor_free_term(real_terms[3]);
+      real_terms[3] = NULL;
+      raptor_parser_error(rdf_parser, "N-Triples only allows 3 terms");
+      goto cleanup;
+    }
+  }
 
-  if(terms[3] && term_types[3] == RAPTOR_TERM_TYPE_LITERAL) {
+  if(real_terms[3] && real_terms[3]->type == RAPTOR_TERM_TYPE_LITERAL) {
     if(!ntriples_parser->literal_graph_warning++)
       raptor_parser_warning(rdf_parser, "Ignoring N-Quad literal contexts");
 
-    terms[3] = NULL;
+    raptor_free_term(real_terms[3]);
+    real_terms[3] = NULL;
   }
 
   raptor_ntriples_generate_statement(rdf_parser, 
-                                     terms[0], term_types[0],
-                                     terms[1], term_types[1],
-                                     terms[2], term_types[2],
-                                     object_literal_language,
-                                     object_literal_datatype,
-                                     terms[3], term_types[3]);
+                                     real_terms[0],
+                                     real_terms[1],
+                                     real_terms[2],
+                                     real_terms[3]);
 
-  rdf_parser->locator.byte += len;
+  rdf_parser->locator.byte += RAPTOR_BAD_CAST(int, len);
 
  cleanup:
 
@@ -864,7 +847,7 @@ raptor_ntriples_parse_chunk(raptor_parser* rdf_parser,
   if(!len)
     return 0;
 
-  buffer = (unsigned char*)RAPTOR_MALLOC(cstring, ntriples_parser->line_length + len + 1);
+  buffer = RAPTOR_MALLOC(unsigned char*, ntriples_parser->line_length + len + 1);
   if(!buffer) {
     raptor_parser_fatal_error(rdf_parser, "Out of memory");
     return 1;
@@ -872,7 +855,7 @@ raptor_ntriples_parse_chunk(raptor_parser* rdf_parser,
 
   if(ntriples_parser->line_length) {
     memcpy(buffer, ntriples_parser->line, ntriples_parser->line_length);
-    RAPTOR_FREE(cstring, ntriples_parser->line);
+    RAPTOR_FREE(char*, ntriples_parser->line);
   }
 
   ntriples_parser->line = buffer;
@@ -945,7 +928,7 @@ raptor_ntriples_parse_chunk(raptor_parser* rdf_parser,
 #endif
   }
 
-  ntriples_parser->offset = start-buffer;
+  ntriples_parser->offset = start - buffer;
 
   len = ntriples_parser->line_length - ntriples_parser->offset;
     
@@ -955,7 +938,7 @@ raptor_ntriples_parse_chunk(raptor_parser* rdf_parser,
 #if defined(RAPTOR_DEBUG) && RAPTOR_DEBUG > 1
     RAPTOR_DEBUG3("collapsing buffer from %d to %d bytes\n", ntriples_parser->line_length, (unsigned int)len);
 #endif
-    buffer = (unsigned char*)RAPTOR_MALLOC(cstring, len + 1);
+    buffer = RAPTOR_MALLOC(unsigned char*, len + 1);
     if(!buffer) {
       raptor_parser_fatal_error(rdf_parser, "Out of memory");
       return 1;
@@ -966,7 +949,7 @@ raptor_ntriples_parse_chunk(raptor_parser* rdf_parser,
            len);
     buffer[len] = '\0';
 
-    RAPTOR_FREE(cstring, ntriples_parser->line);
+    RAPTOR_FREE(char*, ntriples_parser->line);
 
     ntriples_parser->line = buffer;
     ntriples_parser->line_length -= ntriples_parser->offset;
@@ -1012,7 +995,7 @@ raptor_ntriples_parse_start(raptor_parser* rdf_parser)
 }
 
 
-#ifdef RAPTOR_PARSER_NTRIPLES
+#if defined RAPTOR_PARSER_NTRIPLES || defined RAPTOR_PARSER_NQUADS
 static int
 raptor_ntriples_parse_recognise_syntax(raptor_parser_factory* factory, 
                                        const unsigned char *buffer, size_t len,
@@ -1092,14 +1075,16 @@ raptor_ntriples_parse_recognise_syntax(raptor_parser_factory* factory,
 
 static const char* const ntriples_names[2] = { "ntriples", NULL };
 
-static const char* const ntriples_uri_strings[2] = {
+static const char* const ntriples_uri_strings[3] = {
+  "http://www.w3.org/ns/formats/N-Triples",
   "http://www.w3.org/TR/rdf-testcases/#ntriples",
   NULL
 };
   
-#define NTRIPLES_TYPES_COUNT 1
+#define NTRIPLES_TYPES_COUNT 2
 static const raptor_type_q ntriples_types[NTRIPLES_TYPES_COUNT + 1] = {
-  { "text/plain", 10, 1}, 
+  { "application/n-triples", 21, 10},
+  { "text/plain", 10, 1},
   { NULL, 0, 0}
 };
 
@@ -1148,6 +1133,7 @@ raptor_nquads_parse_recognise_syntax(raptor_parser_factory* factory,
                                      const char *mime_type)
 {
   int score = 0;
+  int ntriples_score;
   
   if(suffix) {
     if(!strcmp((const char*)suffix, "nq"))
@@ -1168,7 +1154,11 @@ raptor_nquads_parse_recognise_syntax(raptor_parser_factory* factory,
       score += 2;
   }
   
-  /* Do not guess using content since it looks so similar to N-Triples */
+  /* ntriples is a subset of nquads, score higher than ntriples */
+  ntriples_score = raptor_ntriples_parse_recognise_syntax(factory, buffer, len, identifier, suffix, mime_type);
+  if(ntriples_score > 0) {
+    score += ntriples_score + 1;
+  }
 
   return score;
 }
