@@ -36,16 +36,17 @@ extern "C" {
 
 #ifdef RAPTOR_INTERNAL
 
-#ifdef MAINTAINER_MODE
-#include <git-version.h>
-#endif
-
 /* for the memory allocation functions */
 #ifdef HAVE_STDLIB_H
 #include <stdlib.h>
 #undef HAVE_STDLIB_H
 #endif
 
+/* Some internal functions are needed by the test programs */
+#ifndef RAPTOR_INTERNAL_API
+#define RAPTOR_INTERNAL_API RAPTOR_API
+#endif
+
 /* Can be over-ridden or undefined in a config.h file or -Ddefine */
 #ifndef RAPTOR_INLINE
 #define RAPTOR_INLINE inline
@@ -75,6 +76,10 @@ void raptor_sign_free(void *ptr);
 
 #endif
 
+#ifndef __func__
+#define __func__ "???"
+#endif
+
 #ifdef RAPTOR_DEBUG
 /* Debugging messages */
 #define RAPTOR_DEBUG1(msg) do {fprintf(stderr, "%s:%d:%s: " msg, __FILE__, __LINE__, __func__); } while(0)
@@ -170,17 +175,6 @@ void raptor_sign_free(void *ptr);
 #define MAX_ASCII_INT_SIZE 13
   
 /* XML parser includes */
-#ifdef RAPTOR_XML_EXPAT
-#ifdef HAVE_EXPAT_H
-#include <expat.h>
-#endif
-#ifdef HAVE_XMLPARSE_H
-#include <xmlparse.h>
-#endif
-#endif
-
-
-
 
 #ifdef RAPTOR_XML_LIBXML
 
@@ -208,21 +202,6 @@ extern void raptor_libxml_update_document_locator(raptor_sax2* sax2, raptor_loca
 #endif
 
 
-/* expat-only prototypes */
-
-#ifdef RAPTOR_XML_EXPAT
-/* raptor_expat.c exports */
-extern void raptor_expat_init(raptor_sax2* sax2, raptor_uri *base_uri);
-extern void raptor_expat_update_document_locator(raptor_sax2* sax2, raptor_locator *locator);
-
-/* raptor_parse.c */
-void raptor_xml_unparsed_entity_decl_handler(void *user_data, const unsigned char* entityName, const unsigned char* base, const unsigned char* systemId, const unsigned char* publicId, const unsigned char* notationName);
-int raptor_xml_external_entity_ref_handler(void *user_data, const unsigned char* context, const unsigned char* base, const unsigned char* systemId, const unsigned char* publicId);
-
-/* end of expat-only */
-#endif
-
-
 typedef struct raptor_parser_factory_s raptor_parser_factory;
 typedef struct raptor_serializer_factory_s raptor_serializer_factory;
 typedef struct raptor_id_set_s raptor_id_set;
@@ -334,11 +313,14 @@ typedef enum {
   /* These terms are datatypes (used as a literal datatype URI) */
   RDF_NS_XMLLiteral      = 26,
   RDF_NS_PlainLiteral    = 27, /* http://www.w3.org/TR/rdf-text/ */
+  /* RDF 1.1 datatypes */
+  RDF_NS_HTML            = 28,
+  RDF_NS_langString      = 29,
 
   /* These terms are internal */
   RDF_NS_LAST_SYNTAX_TERM = RDF_NS_datatype,
 
-  RDF_NS_LAST            = RDF_NS_PlainLiteral
+  RDF_NS_LAST            = RDF_NS_langString
 } raptor_rdf_ns_term_id;
 
 
@@ -474,7 +456,7 @@ raptor_namespace** raptor_namespace_stack_to_array(raptor_namespace_stack *nstac
 
 
 /* Size of buffer to use when reading from a file */
-#ifdef BUFSIZ
+#if defined(BUFSIZ) && BUFSIZ > 4096
 #define RAPTOR_READ_BUFFER_SIZE BUFSIZ
 #else
 #define RAPTOR_READ_BUFFER_SIZE 4096
@@ -601,6 +583,9 @@ struct raptor_parser_factory_s {
 
   /* get the current graph (OPTIONAL) - if not implemented, the current graph is always the default (NULL) and start/end graph marks are synthesised */
   raptor_uri* (*get_graph)(raptor_parser* rdf_parser);
+
+  /* get the locator (OPTIONAL) */
+  raptor_locator* (*get_locator)(raptor_parser* rdf_parser);
 };
 
 
@@ -703,24 +688,24 @@ unsigned char* raptor_world_internal_generate_id(raptor_world *world, unsigned c
 #ifdef RAPTOR_DEBUG
 void raptor_stats_print(raptor_parser *rdf_parser, FILE *stream);
 #endif
-const char* raptor_basename(const char *name);
+RAPTOR_INTERNAL_API const char* raptor_basename(const char *name);
 int raptor_term_print_as_ntriples(const raptor_term *term, FILE* stream);
 
 /* raptor_parse.c */
 raptor_parser_factory* raptor_world_get_parser_factory(raptor_world* world, const char *name);  
 void raptor_delete_parser_factories(void);
-const char* raptor_parser_get_accept_header_all(raptor_world* world);
+RAPTOR_INTERNAL_API const char* raptor_parser_get_accept_header_all(raptor_world* world);
 int raptor_parser_set_uri_filter_no_net(void *user_data, raptor_uri* uri);
 void raptor_parser_parse_uri_write_bytes(raptor_www* www, void *userdata, const void *ptr, size_t size, size_t nmemb);
 void raptor_parser_fatal_error(raptor_parser* parser, const char *message, ...) RAPTOR_PRINTF_FORMAT(2, 3);
 void raptor_parser_error(raptor_parser* parser, const char *message, ...) RAPTOR_PRINTF_FORMAT(2, 3);
-void raptor_parser_error_varargs(raptor_parser* parser, const char *message, va_list arguments) RAPTOR_PRINTF_FORMAT(2, 0);
+RAPTOR_INTERNAL_API void raptor_parser_log_error_varargs(raptor_parser* parser, raptor_log_level level, const char *message, va_list arguments) RAPTOR_PRINTF_FORMAT(3, 0);
 void raptor_parser_warning(raptor_parser* parser, const char *message, ...) RAPTOR_PRINTF_FORMAT(2, 3);
 
 /* logging */
 void raptor_world_internal_set_ignore_errors(raptor_world* world, int flag);
 void raptor_log_error_varargs(raptor_world* world, raptor_log_level level, raptor_locator* locator, const char* message, va_list arguments) RAPTOR_PRINTF_FORMAT(4, 0);
-void raptor_log_error_formatted(raptor_world* world, raptor_log_level level, raptor_locator* locator, const char* message, ...) RAPTOR_PRINTF_FORMAT(4, 5);
+RAPTOR_INTERNAL_API void raptor_log_error_formatted(raptor_world* world, raptor_log_level level, raptor_locator* locator, const char* message, ...) RAPTOR_PRINTF_FORMAT(4, 5);
 void raptor_log_error(raptor_world* world, raptor_log_level level, raptor_locator* locator, const char* message);
 
 
@@ -728,7 +713,7 @@ void raptor_log_error(raptor_world* world, raptor_log_level level, raptor_locato
 
 typedef struct raptor_rdfxml_parser_s raptor_rdfxml_parser;
 
-/* Prototypes for common expat/libxml parsing event-handling functions */
+/* Prototypes for common libxml parsing event-handling functions */
 extern void raptor_xml_start_element_handler(void *user_data, const unsigned char *name, const unsigned char **atts);
 extern void raptor_xml_end_element_handler(void *user_data, const unsigned char *name);
 /* s is not 0 terminated. */
@@ -761,8 +746,8 @@ int raptor_check_ordinal(const unsigned char *name);
 #endif
 
 
-/* raptor_nfc.c */
-int raptor_nfc_check (const unsigned char* string, size_t len, int *error);
+/* raptor_nfc_icu.c */
+int raptor_nfc_icu_check (const unsigned char* string, size_t len, int *error);
 
 
 /* raptor_namespace.c */
@@ -807,6 +792,7 @@ void raptor_qname_print(FILE *stream, raptor_qname* name);
 int raptor_uri_init(raptor_world* world);
 void raptor_uri_finish(raptor_world* world);
 raptor_uri* raptor_new_uri_from_rdf_ordinal(raptor_world* world, int ordinal);
+size_t raptor_uri_normalize_path(unsigned char* path_buffer, size_t path_len);
 
 /* parsers */
 int raptor_init_parser_rdfxml(raptor_world* world);
@@ -826,30 +812,44 @@ void raptor_terminate_parser_grddl_common(raptor_world *world);
 
 #ifdef RAPTOR_PARSER_RDFA
 #define rdfa_add_item raptor_librdfa_rdfa_add_item
+#define rdfa_append_to_list_mapping raptor_librdfa_rdfa_append_to_list_mapping
 #define rdfa_canonicalize_string raptor_librdfa_rdfa_canonicalize_string
+#define rdfa_complete_current_property_value_triples raptor_librdfa_rdfa_complete_current_property_value_triples
 #define rdfa_complete_incomplete_triples raptor_librdfa_rdfa_complete_incomplete_triples
+#define rdfa_complete_list_triples raptor_librdfa_rdfa_complete_list_triples
 #define rdfa_complete_object_literal_triples raptor_librdfa_rdfa_complete_object_literal_triples
 #define rdfa_complete_relrev_triples raptor_librdfa_rdfa_complete_relrev_triples
 #define rdfa_complete_type_triples raptor_librdfa_rdfa_complete_type_triples
 #define rdfa_copy_list raptor_librdfa_rdfa_copy_list
+#define rdfa_copy_mapping raptor_librdfa_rdfa_copy_mapping
 #define rdfa_create_bnode raptor_librdfa_rdfa_create_bnode
 #define rdfa_create_context raptor_librdfa_rdfa_create_context
 #define rdfa_create_list raptor_librdfa_rdfa_create_list
+#define rdfa_create_list_mapping raptor_librdfa_rdfa_create_list_mapping
+#define rdfa_create_mapping raptor_librdfa_rdfa_create_mapping
 #define rdfa_create_new_element_context raptor_librdfa_rdfa_create_new_element_context
 #define rdfa_create_triple raptor_librdfa_rdfa_create_triple
-#define rdfa_establish_new_subject raptor_librdfa_rdfa_establish_new_subject
-#define rdfa_establish_new_subject_with_relrev raptor_librdfa_rdfa_establish_new_subject_with_relrev
-#define rdfa_free_context raptor_librdfa_rdfa_free_context
-#define rdfa_free_context_stack raptor_librdfa_rdfa_free_context_stack
+#define rdfa_establish_new_1_0_subject raptor_librdfa_rdfa_establish_new_1_0_subject
+#define rdfa_establish_new_1_0_subject_with_relrev raptor_librdfa_ablish_new_1_0_subject_with_relrev
+#define rdfa_establish_new_1_1_subject raptor_librdfa_ablish_new_1_1_subject
+#define rdfa_establish_new_1_1_subject_with_relrev raptor_librdfa_ablish_new_1_1_subject_with_relrev
+#define rdfa_establish_new_inlist_triples raptor_librdfa_ablish_new_inlist_triples
+#define rdfa_free_context raptor_librdfa_free_context
+#define rdfa_free_context_stack raptor_librdfa_free_context_stack
 #define rdfa_free_list raptor_librdfa_rdfa_free_list
+#define rdfa_free_mapping raptor_librdfa_rdfa_free_mapping
 #define rdfa_free_triple raptor_librdfa_rdfa_free_triple
 #define rdfa_get_buffer raptor_librdfa_rdfa_get_buffer
-#define rdfa_get_curie_type raptor_librdfa_get_curie_type
+#define rdfa_get_curie_type raptor_librdfa_rdfa_get_curie_type
+#define rdfa_get_list_mapping raptor_librdfa_rdfa_get_list_mapping
+#define rdfa_get_mapping raptor_librdfa_rdfa_get_mapping
 #define rdfa_init_base raptor_librdfa_rdfa_init_base
 #define rdfa_init_context raptor_librdfa_rdfa_init_context
 #define rdfa_iri_get_base raptor_librdfa_rdfa_iri_get_base
 #define rdfa_join_string raptor_librdfa_rdfa_join_string
 #define rdfa_n_append_string raptor_librdfa_rdfa_n_append_string
+#define rdfa_names raptor_librdfa_rdfa_names
+#define rdfa_next_mapping raptor_librdfa_rdfa_next_mapping
 #define rdfa_parse raptor_librdfa_rdfa_parse
 #define rdfa_parse_buffer raptor_librdfa_rdfa_parse_buffer
 #define rdfa_parse_chunk raptor_librdfa_rdfa_parse_chunk
@@ -857,19 +857,27 @@ void raptor_terminate_parser_grddl_common(raptor_world *world);
 #define rdfa_parse_start raptor_librdfa_rdfa_parse_start
 #define rdfa_pop_item raptor_librdfa_rdfa_pop_item
 #define rdfa_print_list raptor_librdfa_rdfa_print_list
+#define rdfa_print_mapping raptor_librdfa_rdfa_print_mapping
+#define rdfa_print_string raptor_librdfa_rdfa_print_string
 #define rdfa_print_triple raptor_librdfa_rdfa_print_triple
-#define rdfa_process_doctype raptor_librdfa_rdfa_process_doctype
+#define rdfa_print_triple_list raptor_librdfa_rdfa_print_triple_list
 #define rdfa_push_item raptor_librdfa_rdfa_push_item
+#define rdfa_replace_list raptor_librdfa_rdfa_replace_list
 #define rdfa_replace_string raptor_librdfa_rdfa_replace_string
 #define rdfa_resolve_curie raptor_librdfa_rdfa_resolve_curie
 #define rdfa_resolve_curie_list raptor_librdfa_rdfa_resolve_curie_list
 #define rdfa_resolve_relrev_curie raptor_librdfa_rdfa_resolve_relrev_curie
 #define rdfa_resolve_uri raptor_librdfa_rdfa_resolve_uri
+#define rdfa_save_incomplete_list_triples raptor_librdfa_rdfa_save_incomplete_list_triples
 #define rdfa_save_incomplete_triples raptor_librdfa_rdfa_save_incomplete_triples
 #define rdfa_set_buffer_filler raptor_librdfa_rdfa_set_buffer_filler
 #define rdfa_set_default_graph_triple_handler raptor_librdfa_rdfa_set_default_graph_triple_handler
 #define rdfa_set_processor_graph_triple_handler raptor_librdfa_rdfa_set_processor_graph_triple_handler
+#define rdfa_setup_initial_context raptor_librdfa_rdfa_setup_initial_context
 #define rdfa_update_language raptor_librdfa_rdfa_update_language
+#define rdfa_update_mapping raptor_librdfa_rdfa_update_mapping
+#define rdfa_update_uri_mappings raptor_librdfa_rdfa_update_uri_mappings
+#define rdfa_uri_strings raptor_librdfa_rdfa_uri_strings
 #endif
 
 /* raptor_parse.c */
@@ -888,8 +896,8 @@ int raptor_init_serializer_atom(raptor_world* world);
 extern const unsigned char * const raptor_atom_namespace_uri;
 
 /* raptor_rfc2396.c */
-raptor_uri_detail* raptor_new_uri_detail(const unsigned char *uri_string);
-void raptor_free_uri_detail(raptor_uri_detail* uri_detail);
+RAPTOR_INTERNAL_API raptor_uri_detail* raptor_new_uri_detail(const unsigned char *uri_string);
+RAPTOR_INTERNAL_API void raptor_free_uri_detail(raptor_uri_detail* uri_detail);
 unsigned char* raptor_uri_detail_to_string(raptor_uri_detail *ud, size_t* len_p);
 
 /* serializers */
@@ -924,7 +932,7 @@ extern const raptor_unichar raptor_unicode_max_codepoint;
 
 int raptor_unicode_is_namestartchar(raptor_unichar c);
 int raptor_unicode_is_namechar(raptor_unichar c);
-int raptor_unicode_check_utf8_nfc_string(const unsigned char *input, size_t length);
+int raptor_unicode_check_utf8_nfc_string(const unsigned char *input, size_t length, int* error);
 
 /* raptor_www*.c */
 #ifdef RAPTOR_WWW_LIBXML
@@ -1013,15 +1021,16 @@ void raptor_www_curl_init(raptor_www *www);
 void raptor_www_curl_free(raptor_www *www);
 int raptor_www_curl_fetch(raptor_www *www);
 int raptor_www_curl_set_ssl_cert_options(raptor_www* www, const char* cert_filename, const char* cert_type, const char* cert_passphrase);
+int raptor_www_curl_set_ssl_verify_options(raptor_www* www, int verify_peer, int verify_host);
 
 void raptor_www_libfetch_init(raptor_www *www);
 void raptor_www_libfetch_free(raptor_www *www);
 int raptor_www_libfetch_fetch(raptor_www *www);
 
 /* raptor_set.c */
-raptor_id_set* raptor_new_id_set(raptor_world* world);
-void raptor_free_id_set(raptor_id_set* set);
-int raptor_id_set_add(raptor_id_set* set, raptor_uri* base_uri, const unsigned char *item, size_t item_len);
+RAPTOR_INTERNAL_API raptor_id_set* raptor_new_id_set(raptor_world* world);
+RAPTOR_INTERNAL_API void raptor_free_id_set(raptor_id_set* set);
+RAPTOR_INTERNAL_API int raptor_id_set_add(raptor_id_set* set, raptor_uri* base_uri, const unsigned char *item, size_t item_len);
 #if defined(RAPTOR_DEBUG) && RAPTOR_DEBUG > 1
 void raptor_id_set_stats_print(raptor_id_set* set, FILE *stream);
 #endif
@@ -1064,12 +1073,6 @@ struct raptor_sax2_s {
   raptor_world* world;
   void* user_data;
   
-#ifdef RAPTOR_XML_EXPAT
-  XML_Parser xp;
-#ifdef EXPAT_UTF8_BOM_CRASH
-  int tokens_count; /* used to see if trying to get location info is safe */
-#endif
-#endif
 #ifdef RAPTOR_XML_LIBXML
   /* structure holding sax event handlers */
   xmlSAXHandler sax;
@@ -1122,6 +1125,12 @@ struct raptor_sax2_s {
 
   /* sax2 init failed - do not try to do anything with it */
   int failed;
+  
+  /* call SAX2 handlers if non-0 */
+  int enabled;
+
+  void* uri_filter_user_data;
+  raptor_uri_filter_func uri_filter;
 };
 
 int raptor_sax2_init(raptor_world* world);
@@ -1147,7 +1156,7 @@ void raptor_sax2_cdata(void* user_data, const unsigned char *s, int len);
 void raptor_sax2_comment(void* user_data, const unsigned char *s);
 void raptor_sax2_unparsed_entity_decl(void* user_data, const unsigned char* entityName, const unsigned char* base, const unsigned char* systemId, const unsigned char* publicId, const unsigned char* notationName);
 int raptor_sax2_external_entity_ref(void* user_data, const unsigned char* context, const unsigned char* base, const unsigned char* systemId, const unsigned char* publicId);
-
+int raptor_sax2_check_load_uri_string(raptor_sax2* sax2, const unsigned char* uri_string);
 
 /* turtle_parser.y and turtle_lexer.l */
 typedef struct raptor_turtle_parser_s raptor_turtle_parser;
@@ -1182,11 +1191,11 @@ struct raptor_uri_detail_s
 
 
 /* for time_t */
-#if TIME_WITH_SYS_TIME
+#ifdef TIME_WITH_SYS_TIME
 # include <sys/time.h>
 # include <time.h>
 #else
-# if HAVE_SYS_TIME_H
+# ifdef HAVE_SYS_TIME_H
 #  include <sys/time.h>
 # else
 #  include <time.h>
@@ -1214,7 +1223,8 @@ typedef void (*raptor_simple_message_handler)(void *user_data, const char *messa
 
 
 /* turtle_common.c */
-int raptor_stringbuffer_append_turtle_string(raptor_stringbuffer* stringbuffer, const unsigned char *text, size_t len, int delim, raptor_simple_message_handler error_handler, void *error_data);
+RAPTOR_INTERNAL_API int raptor_stringbuffer_append_turtle_string(raptor_stringbuffer* stringbuffer, const unsigned char *text, size_t len, int delim, raptor_simple_message_handler error_handler, void *error_data);
+RAPTOR_INTERNAL_API int raptor_turtle_check_uri_string(unsigned char *string);
 
 
 /* raptor_abbrev.c */
@@ -1276,26 +1286,28 @@ raptor_qname* raptor_new_qname_from_resource(raptor_sequence* namespaces, raptor
 typedef struct raptor_turtle_writer_s raptor_turtle_writer;
 
 /* Turtle Writer Class (raptor_turtle_writer) */
-raptor_turtle_writer* raptor_new_turtle_writer(raptor_world* world, raptor_uri* base_uri, int write_base_uri, raptor_namespace_stack *nstack, raptor_iostream* iostr);
-void raptor_free_turtle_writer(raptor_turtle_writer* turtle_writer);
-void raptor_turtle_writer_raw(raptor_turtle_writer* turtle_writer, const unsigned char *s);
-void raptor_turtle_writer_raw_counted(raptor_turtle_writer* turtle_writer, const unsigned char *s, unsigned int len);
-void raptor_turtle_writer_namespace_prefix(raptor_turtle_writer* turtle_writer, raptor_namespace* ns);
+RAPTOR_INTERNAL_API raptor_turtle_writer* raptor_new_turtle_writer(raptor_world* world, raptor_uri* base_uri, int write_base_uri, raptor_namespace_stack *nstack, raptor_iostream* iostr);
+RAPTOR_INTERNAL_API void raptor_free_turtle_writer(raptor_turtle_writer* turtle_writer);
+RAPTOR_INTERNAL_API void raptor_turtle_writer_raw(raptor_turtle_writer* turtle_writer, const unsigned char *s);
+RAPTOR_INTERNAL_API void raptor_turtle_writer_raw_counted(raptor_turtle_writer* turtle_writer, const unsigned char *s, unsigned int len);
+RAPTOR_INTERNAL_API void raptor_turtle_writer_namespace_prefix(raptor_turtle_writer* turtle_writer, raptor_namespace* ns);
 void raptor_turtle_writer_base(raptor_turtle_writer* turtle_writer, raptor_uri* base_uri);
-void raptor_turtle_writer_increase_indent(raptor_turtle_writer *turtle_writer);
-void raptor_turtle_writer_decrease_indent(raptor_turtle_writer *turtle_writer);
-void raptor_turtle_writer_newline(raptor_turtle_writer *turtle_writer);
-void raptor_turtle_writer_reference(raptor_turtle_writer* turtle_writer, raptor_uri* uri);
-int raptor_turtle_writer_literal(raptor_turtle_writer* turtle_writer, raptor_namespace_stack *nstack, const unsigned char *s, const unsigned char* lang, raptor_uri* datatype);
-void raptor_turtle_writer_qname(raptor_turtle_writer* turtle_writer, raptor_qname* qname);
-int raptor_turtle_writer_quoted_counted_string(raptor_turtle_writer* turtle_writer, const unsigned char *s, size_t length);
+RAPTOR_INTERNAL_API void raptor_turtle_writer_increase_indent(raptor_turtle_writer *turtle_writer);
+RAPTOR_INTERNAL_API void raptor_turtle_writer_decrease_indent(raptor_turtle_writer *turtle_writer);
+RAPTOR_INTERNAL_API void raptor_turtle_writer_newline(raptor_turtle_writer *turtle_writer);
+RAPTOR_INTERNAL_API int raptor_turtle_writer_reference(raptor_turtle_writer* turtle_writer, raptor_uri* uri);
+RAPTOR_INTERNAL_API int raptor_turtle_writer_literal(raptor_turtle_writer* turtle_writer, raptor_namespace_stack *nstack, const unsigned char *s, const unsigned char* lang, raptor_uri* datatype);
+RAPTOR_INTERNAL_API void raptor_turtle_writer_qname(raptor_turtle_writer* turtle_writer, raptor_qname* qname);
+RAPTOR_INTERNAL_API int raptor_turtle_writer_quoted_counted_string(raptor_turtle_writer* turtle_writer, const unsigned char *s, size_t length);
 void raptor_turtle_writer_comment(raptor_turtle_writer* turtle_writer, const unsigned char *s);
-int raptor_turtle_writer_set_option(raptor_turtle_writer *turtle_writer, raptor_option option, int value);
+RAPTOR_INTERNAL_API int raptor_turtle_writer_set_option(raptor_turtle_writer *turtle_writer, raptor_option option, int value);
 int raptor_turtle_writer_set_option_string(raptor_turtle_writer *turtle_writer, raptor_option option, const unsigned char *value);
 int raptor_turtle_writer_get_option(raptor_turtle_writer *turtle_writer, raptor_option option);
 const unsigned char *raptor_turtle_writer_get_option_string(raptor_turtle_writer *turtle_writer, raptor_option option);
 void raptor_turtle_writer_bnodeid(raptor_turtle_writer* turtle_writer, const unsigned char *bnodeid, size_t len);
-
+int raptor_turtle_writer_uri(raptor_turtle_writer* turtle_writer, raptor_uri* uri);
+int raptor_turtle_writer_term(raptor_turtle_writer* turtle_writer, raptor_term* term);
+int raptor_turtle_is_legal_turtle_qname(raptor_qname* qname);
 
 /**
  * raptor_json_writer:
@@ -1327,7 +1339,7 @@ int raptor_rdfxmla_serialize_set_single_node(raptor_serializer* serializer, rapt
 int raptor_rdfxmla_serialize_set_write_typed_nodes(raptor_serializer* serializer, int value);
 
 /* snprintf.c */
-int raptor_format_integer(char* buffer, size_t bufsize, int integer, unsigned int base, int width, char padding);
+size_t raptor_format_integer(char* buffer, size_t bufsize, int integer, unsigned int base, int width, char padding);
 
 /* raptor_world structure */
 #define RAPTOR1_WORLD_MAGIC_1 0
@@ -1341,7 +1353,7 @@ int raptor_format_integer(char* buffer, size_t bufsize, int integer, unsigned in
   } while(0)  
     
 
-int raptor_check_world_internal(raptor_world* world, const char* name);
+RAPTOR_INTERNAL_API int raptor_check_world_internal(raptor_world* world, const char* name);
 
 
 
