@@ -3,7 +3,7 @@
  * raptor_serialize_turtle.c - Turtle serializer
  *
  * Copyright (C) 2006,2008 Dave Robillard
- * Copyright (C) 2004-2010 David Beckett http://www.dajobe.org/
+ * Copyright (C) 2004-2013 David Beckett http://www.dajobe.org/
  * Copyright (C) 2004-2005 University of Bristol, UK http://www.bristol.ac.uk/
  * Copyright (C) 2005 Steve Shepard steveshep@gmail.com
  * 
@@ -27,11 +27,6 @@
 #include <raptor_config.h>
 #endif
 
-#ifdef WIN32
-#include <win32_raptor_config.h>
-#endif
-
-
 #include <stdio.h>
 #include <string.h>
 #include <ctype.h>
@@ -126,7 +121,7 @@ static int raptor_turtle_serialize_end(raptor_serializer* serializer);
 static void raptor_turtle_serialize_finish_factory(raptor_serializer_factory* factory);
 
 
-static int
+int
 raptor_turtle_is_legal_turtle_qname(raptor_qname* qname)
 {
   const char* prefix_name;
@@ -664,6 +659,7 @@ raptor_turtle_emit_subject(raptor_serializer *serializer,
     if(rc)
       return rc;
     blank = 0;
+    collection = 0;
     
   } else if(subject->node->term->type == RAPTOR_TERM_TYPE_BLANK) {
     if((subject->node->count_as_subject == 1 && 
@@ -1191,3 +1187,219 @@ raptor_init_serializer_turtle(raptor_world* world)
                                              &raptor_turtle_serializer_register_factory);
 }
 
+
+
+/**
+ * raptor_uri_turtle_write:
+ * @world: world
+ * @iostr: iostream for writing
+ * @uri: uri
+ * @nstack: namespace stack
+ * @base_uri: base URI
+ *
+ * Write #raptor_uri to a stream in turtle syntax (using QNames).
+ *
+ * Note: This creates and destroys several internal objects for each
+ * call so for more efficient writing, create a turtle serializer.
+ *
+ * Return value: non-0 on failure
+ */
+int
+raptor_uri_turtle_write(raptor_world *world,
+                        raptor_iostream* iostr,
+                        raptor_uri* uri, 
+                        raptor_namespace_stack *nstack,
+                        raptor_uri *base_uri)
+{
+  int rc;
+  raptor_turtle_writer* turtle_writer;
+  
+  turtle_writer = raptor_new_turtle_writer(world, base_uri, 0, nstack, iostr);
+  if(!turtle_writer)
+    return 1;
+
+  rc = raptor_turtle_writer_uri(turtle_writer, uri);
+  
+  raptor_free_turtle_writer(turtle_writer);
+  
+  return rc;
+}
+
+
+
+/**
+ * raptor_term_turtle_write:
+ * @iostr: iostream for writing
+ * @term: term
+ * @nstack: namespace stack
+ * @base_uri: base URI
+ *
+ * Write #raptor_term to a stream in turtle syntax (using QNames).
+ *
+ * Note: This creates and destroys several internal objects for each
+ * call so for more efficient writing, create a turtle serializer.
+ *
+ * Return value: non-0 on failure
+ */
+int
+raptor_term_turtle_write(raptor_iostream* iostr,
+                         raptor_term* term, 
+                         raptor_namespace_stack *nstack,
+                         raptor_uri *base_uri)
+{
+  int rc;
+  raptor_turtle_writer* turtle_writer;
+  
+  turtle_writer = raptor_new_turtle_writer(term->world, base_uri, 0, nstack,
+                                           iostr);
+  if(!turtle_writer)
+    return 1;
+
+  rc = raptor_turtle_writer_term(turtle_writer, term);
+  
+  raptor_free_turtle_writer(turtle_writer);
+  
+  return rc;
+}
+
+
+
+/**
+ * raptor_uri_to_turtle_counted_string:
+ * @world: world
+ * @uri: uri
+ * @nstack: namespace stack
+ * @base_uri: base URI
+ * @len_p: Pointer to location to store length of new string (if not NULL)
+ *
+ * Convert #raptor_uri to a string.
+ * Caller has responsibility to free the string.
+ *
+ * Note: This creates and destroys several internal objects for each
+ * call so for more efficient writing, create a turtle serializer.
+ *
+ * Return value: the new string or NULL on failure.  The length of
+ * the new string is returned in *@len_p if len_p is not NULL.
+ */
+unsigned char*
+raptor_uri_to_turtle_counted_string(raptor_world *world,
+                                    raptor_uri* uri, 
+                                    raptor_namespace_stack *nstack,
+                                    raptor_uri *base_uri,
+                                    size_t *len_p)
+{
+  int rc = 1;
+  raptor_iostream* iostr;
+  unsigned char *s = NULL;
+  raptor_turtle_writer* turtle_writer;
+
+  iostr = raptor_new_iostream_to_string(world,
+                                        (void**)&s, len_p, malloc);
+  if(!iostr)
+    return NULL;
+  
+  turtle_writer = raptor_new_turtle_writer(world, base_uri, 0, nstack, iostr);
+  if(!turtle_writer)
+    goto tidy;
+
+  rc = raptor_turtle_writer_uri(turtle_writer, uri);
+
+  raptor_free_turtle_writer(turtle_writer);
+
+  tidy:
+  raptor_free_iostream(iostr);
+
+  if(rc) {
+    free(s);
+    s = NULL;
+  }
+  
+  return s;
+}
+
+/**
+ * raptor_uri_to_turtle_string:
+ * @world: world
+ * @uri: uri
+ * @nstack: namespace stack
+ * @base_uri: base URI
+ *
+ * Convert #raptor_uri to a string.
+ * Caller has responsibility to free the string.
+ *
+ * Note: This creates and destroys several internal objects for each
+ * call so for more efficient writing, create a turtle serializer.
+ *
+ * Return value: the new string or NULL on failure.
+ */
+unsigned char*
+raptor_uri_to_turtle_string(raptor_world *world,
+                            raptor_uri* uri, 
+                            raptor_namespace_stack *nstack,
+                            raptor_uri *base_uri)
+{
+  return raptor_uri_to_turtle_counted_string(world, uri, nstack, base_uri, NULL);
+}
+
+
+
+/**
+ * raptor_term_to_turtle_counted_string:
+ * @term: term
+ * @nstack: namespace stack
+ * @base_uri: base URI
+ * @len_p: Pointer to location to store length of new string (if not NULL)
+ *
+ * Convert #raptor_term to a string.
+ * Caller has responsibility to free the string.
+ *
+ * Note: This creates and destroys several internal objects for each
+ * call so for more efficient writing, create a turtle serializer.
+ *
+ * Return value: the new string or NULL on failure.  The length of
+ * the new string is returned in *@len_p if len_p is not NULL.
+ */
+unsigned char*
+raptor_term_to_turtle_counted_string(raptor_term* term, 
+                                     raptor_namespace_stack *nstack,
+                                     raptor_uri *base_uri,
+                                     size_t *len_p)
+{
+  int rc;
+  raptor_iostream* iostr;
+  unsigned char *s;
+  iostr = raptor_new_iostream_to_string(term->world,
+                                        (void**)&s, len_p, malloc);
+  if(!iostr)
+    return NULL;
+
+  rc = raptor_term_turtle_write(iostr, term, nstack, base_uri);
+
+  raptor_free_iostream(iostr);
+  if(rc) {
+    free(s);
+    s = NULL;
+  }
+  
+  return s;
+}
+
+/**
+ * raptor_term_to_turtle_string:
+ * @term: term
+ * @nstack: namespace stack
+ * @base_uri: base URI
+ *
+ * Convert #raptor_term to a string.
+ * Caller has responsibility to free the string.
+ *
+ * Return value: the new string or NULL on failure.
+ */
+unsigned char*
+raptor_term_to_turtle_string(raptor_term* term, 
+                             raptor_namespace_stack *nstack,
+                             raptor_uri *base_uri)
+{
+  return raptor_term_to_turtle_counted_string(term, nstack, base_uri, NULL);
+}
+
