@@ -24,11 +24,6 @@
 #include <raptor_config.h>
 #endif
 
-#ifdef WIN32
-#include <win32_raptor_config.h>
-#endif
-
-
 #include <stdio.h>
 #include <string.h>
 #include <ctype.h>
@@ -265,20 +260,21 @@ raptor_new_term_from_literal(raptor_world* world,
                              const unsigned char* language)
 {
   size_t literal_len = 0;
-  unsigned char language_len = 0;
+  size_t language_len = 0;
   
   RAPTOR_CHECK_CONSTRUCTOR_WORLD(world);
 
   raptor_world_open(world);
 
   if(literal)
-    literal_len = strlen((const char*)literal);
+    literal_len = strlen(RAPTOR_GOOD_CAST(const char*, literal));
 
   if(language)
-    language_len = strlen((const char*)language);
-  
+    language_len = strlen(RAPTOR_GOOD_CAST(const char*, language));
+
   return raptor_new_term_from_counted_literal(world, literal, literal_len,
-                                              datatype, language, language_len);
+                                              datatype, language,
+                                              RAPTOR_BAD_CAST(unsigned char, language_len));
 }
 
 
@@ -476,7 +472,8 @@ raptor_term_to_counted_string(raptor_term *term, size_t* len_p)
                                         &string, len_p, NULL);
   if(!iostr)
     return NULL;
-  rc = raptor_term_ntriples_write(term, iostr);
+
+  rc = raptor_term_escaped_write(term, 0, iostr);
   raptor_free_iostream(iostr);
   
   if(rc) {
@@ -523,7 +520,7 @@ raptor_term_print_as_ntriples(const raptor_term *term, FILE* stream)
   if(!iostr)
     return 1;
   
-  rc = raptor_term_ntriples_write(term, iostr);
+  rc = raptor_term_escaped_write(term, 0, iostr);
 
   raptor_free_iostream(iostr);
   
