@@ -27,10 +27,6 @@
 #include <raptor_config.h>
 #endif
 
-#ifdef WIN32
-#include <win32_raptor_config.h>
-#endif
-
 #include <stdio.h>
 #include <string.h>
 #include <ctype.h>
@@ -657,8 +653,9 @@ raptor_uri_filename_to_uri_string(const char *filename)
 #ifndef WIN32
   char *path = NULL;
 #endif
-  /*     "file://" ... \0 */
-  size_t len = 7 + 1;
+  /* "file://" */
+#define RAPTOR_LEN_FILE_CSS 7 
+  size_t len = RAPTOR_LEN_FILE_CSS;
   
   if(!filename)
     return NULL;
@@ -731,7 +728,7 @@ raptor_uri_filename_to_uri_string(const char *filename)
       len += 2; /* strlen(%xx)-1 */
   }
 
-  buffer = RAPTOR_MALLOC(unsigned char*, len);
+  buffer = RAPTOR_MALLOC(unsigned char*, len + 1);
   if(!buffer)
     goto path_done;
 
@@ -768,6 +765,10 @@ raptor_uri_filename_to_uri_string(const char *filename)
 
   path_done:
 #ifndef WIN32
+  /* Normalize the resulting URI path after the "file://"  */
+  raptor_uri_normalize_path(buffer + RAPTOR_LEN_FILE_CSS,
+                            len - RAPTOR_LEN_FILE_CSS);
+
   if(path)
     free(path);
 #endif
@@ -1663,6 +1664,80 @@ raptor_uri_file_exists(raptor_uri* uri)
   return raptor_uri_filename_exists(uri_string + 6);
 }
 
+
+
+/**
+ * raptor_uri_escaped_write:
+ * @uri: uri to write
+ * @base_uri: base uri to write relative to (or NULL)
+ * @flags: bit flags - see #raptor_escaped_write_bitflags
+ * @iostr: raptor iostream
+ * 
+ * Write a #raptor_uri formatted with escapes to a #raptor_iostream
+ * 
+ * Return value: non-0 on failure
+ **/
+int
+raptor_uri_escaped_write(raptor_uri* uri,
+                         raptor_uri* base_uri,
+                         unsigned int flags,
+                         raptor_iostream *iostr)
+{
+  unsigned char *uri_str;
+  int uri_str_owned = 0;
+  size_t len;
+
+  if(!uri)
+    return 1;
+  
+  raptor_iostream_write_byte('<', iostr);
+  if(base_uri) {
+    uri_str = raptor_uri_to_relative_counted_uri_string(base_uri, uri, &len);
+    if(!uri_str)
+      return 1;
+
+    uri_str_owned = 1;
+  } else {
+    uri_str = raptor_uri_as_counted_string(uri, &len);
+  }
+  if(uri_str)
+    raptor_string_escaped_write(uri_str, len, '>', flags, iostr);
+  raptor_iostream_write_byte('>', iostr);
+
+  if(uri_str_owned && uri_str)
+    RAPTOR_FREE(char*, uri_str);
+
+  return 0;
+}
+
+
+/**
+ * raptor_uri_uri_string_is_absolute:
+ * @uri_string: uri to check write
+ * 
+ * Check if a uri string is an absolute URI
+ * 
+ * Return value: >0 if absolute, 0 if not, < 0 on failure
+ **/
+int
+raptor_uri_uri_string_is_absolute(const unsigned char* uri_string)
+{
+  raptor_uri_detail *detail = NULL;
+  int rc = -1;
+  
+  detail = raptor_new_uri_detail(uri_string);
+  if(!detail)
+    goto tidy;
+
+  rc = (detail->scheme_len) > 0;
+  
+  tidy:
+  raptor_free_uri_detail(detail);
+  
+  return rc;
+}
+
+
 #endif /* !STANDALONE */
 
 
